from inverter_characteristics import Piecewise

# stable inverter (ideal)
fwd_fake_step = Piecewise(
    [
        [ 0.0, 0.0 ],
        [ 0.4, 0.0 ],
        [ 0.6, 1.0 ],
        [ 1.0, 1.0 ],
    ]
)

# stable inverter (amplifying)
fwd_fake_1_5x = Piecewise(
    [
        [ 0.0,  0.0 ],
        [ 0.65, 1.0 ],
        [ 1.0,  1.0 ],
    ]
)
# stable inverter (amplifying only from 0.3 -> 0.5)
fwd_fake_slope_change_before_0_5 = Piecewise(
    [
        [ 0.0, 0.2 ],
        [ 0.3, 0.3 ],
        [ 0.5, 0.6 ],
        [ 1.0, 1.0 ],
    ]
)
# failed inverter (>1.0 slope happens too late)
# flipping x doesn't fix.
# however, shifting x by -0.1 and y by -0.2 and *then* inverting x does.
# - this gives us a concave-up 1-x like curve
fwd_fake_slope_change_after_0_5 = Piecewise(
    [
        [ 0.0, 0.2 ],
        [ 0.3, 0.3 ],
        [ 0.6, 0.5 ],
        [ 1.0, 1.0 ],
    ]
)

slope_fake_hill = [
  0.8, 0.9, 1.0, 1.1, 1.2, 1.2, 1.1, 1.0, 0.9, 0.8
]
fwd_fake_hill = Piecewise(
    [ (0.1*i, 0.1 * sum(slope_fake_hill[0:i])) for i in range(11) ]
)

fwd_fake_asymmetric_hill = Piecewise(
    [
        (0.0, 0.20),
        (0.2, 0.30),
        (0.4, 0.45),
        (0.6, 0.75),
        (0.8, 0.80),
        (1.0, 0.85),
    ]
)
# valid inverter; the [0.6, 1.0] -> 0.8 mapping *fixes* the logic low value to
#   1.0 - 0.8 = 0.2
#   and allows anything 0.6 to 1.0 to be recognized as logic high immediately.
# i.e. "bottoming out" is a *good* thing
fwd_fake_asymmetric_flats = Piecewise(
    [
        (0.0, 0.20),
        (0.2, 0.30),
        (0.6, 0.80),
        (1.0, 0.80),
    ]
)

fwd_fake_asymmetric_overdrive = Piecewise(
    [
        (0.0, 0.40),
        (0.3, 0.50),
        (0.6, 0.85),
        (1.0, 0.90),
    ]
)

fwd_fake_asymmetric_bottom_out = Piecewise(
    [
        (0.0, 0.00),
        (0.8, 0.99),
        (1.0, 1.00),
    ]
)

