#!/usr/bin/env python3

from natsort import natsorted

from stacked_cores_52xx import *
from stacked_cores_52xx_plotters import *


def extract_58xx_tx(meas_rows: list) -> tuple:
    """
    extracts a flat tuple of input/output M mappings from a 58xx run
    """

    return (
        meas_rows[0].m[0],  # input (neg)
        meas_rows[0].m[2],  # I/O (pos)
        meas_rows[0].m[3],  # I/O (neg)
        meas_rows[0].m[5],  # input (pos)
        meas_rows[1].m[2],  # output (pos)
        meas_rows[1].m[3],  # output (neg)
        0.5 * (meas_rows[1].m[2] - meas_rows[1].m[3])  # output (diff)
    )


buf_gates = read_db(lambda name: name.startswith("58-"))

sweep_buf_inputs = lambda points=101: [(-m, m, -m, m, None, None, None) for m in sweep_1d(points)]
sweep_mpos = lambda mneg, points=101: [(mneg, m, mneg, m, None, None, None) for m in sweep_1d(points)]


for name, meas in natsorted(buf_gates.items()):
    # sweep Mneg = -Mpos
    # trace = eval_series(meas, sweep_buf_inputs(41), extract_58xx_tx, y_idx=6)
    # plot(f"{name}", "Mpos", trace)
    # plot_slope(f"slope {name}", "Mpos", trace)

    # sweep M0 with M1 fixed constant (to check for some `max(M+, M-)`-like effect
    trace = eval_series(meas, sweep_mpos(-5000, 41), extract_58xx_tx, y_idx=6)
    plot(f"{name}", "Mneg=-5000", trace)
    plot_slope(f"slope {name}", "Mneg=-5000", trace)
    trace = eval_series(meas, sweep_mpos(5000, 41), extract_58xx_tx, y_idx=6)
    plot(f"{name}", "Mneg=5000", trace)
    plot_slope(f"slope {name}", "Mneg=5000", trace)
