#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 12-xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_12xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_start = last_row_before_t(rows, 2e-9)
    tx_end = last_row_before_t(rows, 3e-9)
    noop_start = last_row_before_t(rows, 5e-9)
    noop_end = last_row_before_t(rows, 6e-9)

    m_tx_start = extract_m(tx_start)
    m_tx_end = extract_m(tx_end)
    m_noop_start = extract_m(noop_start)
    m_noop_end = extract_m(noop_end)

    m1_tx = abs(m_tx_end[1] - m_tx_start[1])
    m1_noop = abs(m_noop_end[1] - m_noop_start[1])

    m_tx_arr = [round(abs(m_tx_end[i] - m_tx_start[i])) for i in [0, 2]]
    m_tx = sum(m_tx_arr)
    m_noop_arr = [round(abs(m_noop_end[i] - m_noop_start[i])) for i in [0, 2]]
    m_noop = sum(m_noop_arr)

    ratio_tx_noop = m_tx / m_noop
    ratio_tx_m1 = m_tx / m1_tx
    ratio_tx_noop_m1 = (m_tx - m_noop) / m1_tx

    print(f'm1 tx: {m1_tx}  ({m_tx_start[1]} -> {m_tx_end[1]})')
    print(f'm1 noop: {m1_noop}  ({m_noop_start[1]} -> {m_noop_end[1]})')
    print('')
    print(f'm(tx):   {m_tx_start}')
    print(f'      -> {m_tx_end}')
    print('')
    print(f'm(noop): {m_noop_start}')
    print(f'      -> {m_noop_end}')
    print('')
    print(f'tx/noop: {ratio_tx_noop:.3}')
    print(f'tx/m1: {ratio_tx_m1:.3}')
    print(f'(tx-noop)/m1: {ratio_tx_noop_m1:.3}')

if __name__ == '__main__':
    extract_12xx(sys.argv[1])
