#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 17-xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_17xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_start_0 = last_row_before_t(rows, 2e-9)
    tx_start_1 = last_row_before_t(rows, 4e-9)
    tx_end = last_row_before_t(rows, 5e-9)
    m_tx_start_0 = extract_m(tx_start_0)
    m_tx_start_1 = extract_m(tx_start_1)
    m_tx_end = extract_m(tx_end)

    m0_switch = abs(m_tx_start_1[0] - m_tx_start_0[0])
    m_middle_switch = sum(abs(e - s) for (e, s) in zip(m_tx_start_1[1:-1], m_tx_start_0[1:-1]))
    m_middle_clear = sum(abs(e - s) for (e, s) in zip(m_tx_end[1:-1], m_tx_start_1[1:-1]))
    m_last_switch = abs(m_tx_end[-1] - m_tx_start_1[-1])

    print(f'm0: {m0_switch} ({m_tx_start_0[0]} -> {m_tx_start_1[0]})')
    print(f'm_middle: {m_middle_switch}')
    print(f'm_middle_clear: {m_middle_clear}')
    print(f'm_last: {m_last_switch} ({m_tx_start_1[-1]} -> {m_tx_end[-1]})')
    print('')
    print(f'm:   {m_tx_start_0}')
    print(f'  -> {m_tx_start_1}')
    print(f'  -> {m_tx_end}')

if __name__ == '__main__':
    extract_17xx(sys.argv[1])
