#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 18-xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_18xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_init = last_row_before_t(rows, 2e-9)
    tx_fwd = last_row_before_t(rows, 4e-9)
    tx_rev = last_row_before_t(rows, 6e-9)
    m_init = extract_m(tx_init)
    m_fwd = extract_m(tx_fwd)
    m_rev = extract_m(tx_rev)

    m0_fwd = abs(m_init[0] - m_fwd[0])
    m0_rev = abs(m_rev[0] - m_fwd[0])
    m_rest_fwd = sum(e - s for (e, s) in zip(m_fwd[1:], m_init[1:]))
    m_rest_rev = sum(s - e for (e, s) in zip(m_rev[1:], m_fwd[1:]))

    print(f'\t- m0: {m_init[0]} -> {m_fwd[0]} -> {m_rev[0]}')
    print(f'\t- m0 fwd: {m0_fwd}')
    print(f'\t- m0 rev: {m0_rev}')
    print(f'\t- m_middle fwd: {m_rest_fwd}')
    print(f'\t- m_middle rev: {m_rest_rev}')
    print(f'\t- m:   {m_init}')
    print(f'\t    -> {m_fwd}')
    print(f'\t    -> {m_rev}')

if __name__ == '__main__':
    extract_18xx(sys.argv[1])
