#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 37-xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_37xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_init = last_row_before_t(rows, 2e-9)
    tx_fini = last_row_before_t(rows, 3e-9)
    m_init = extract_m(tx_init)
    m_fini = extract_m(tx_fini)

    m0 = -(m_fini[0] - m_init[0])
    m1 = -(m_fini[1] - m_init[1])
    m2 = -(m_fini[2] - m_init[2])

    m3 = m_fini[3] - m_init[3]

    m4 = m_fini[4] - m_init[4]
    m5 = m_fini[5] - m_init[5]
    m6 = m_fini[6] - m_init[6]

    print(f'\t- madj: {m0 + m1 + m2 - m4 - m5 - m6}')
    print(f'\t\t- m0: {m_init[0]} -> {m_fini[0]}')
    print(f'\t\t- m1: {m_init[1]} -> {m_fini[1]}')
    print(f'\t\t- m2: {m_init[2]} -> {m_fini[2]}')
    print(f'\t\t- m4: {m_init[4]} -> {m_fini[4]}')
    print(f'\t\t- m5: {m_init[5]} -> {m_fini[5]}')
    print(f'\t\t- m6: {m_init[6]} -> {m_fini[6]}')
    print(f'\t- m3: {m3}')
    print(f'\t\t- {m_init[3]} -> {m_fini[3]}')

if __name__ == '__main__':
    extract_37xx(sys.argv[1])
