#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 38-xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_38xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_init = last_row_before_t(rows, 2e-9)
    tx_fini = last_row_before_t(rows, 3e-9)
    m_init = extract_m(tx_init)
    m_fini = extract_m(tx_fini)

    m0 = -(m_fini[0] - m_init[0])
    madj = sum(init - fini for (init, fini) in zip(m_init[1:], m_fini[1:]))

    print(f'\t- madj: {madj}')
    for i, (init, fini) in enumerate(zip(m_init[1:], m_fini[1:])):
        print(f'\t\t- m{i+1}: {init} -> {fini}')
    print(f'\t- m0: {m0}')
    print(f'\t\t- {m_init[0]} -> {m_fini[0]}')

if __name__ == '__main__':
    extract_38xx(sys.argv[1])
