#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 8xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_8xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_start = last_row_before_t(rows, 2e-9)
    tx_end = last_row_before_t(rows, 3e-9)
    noop_start = last_row_before_t(rows, 5e-9)
    noop_end = last_row_before_t(rows, 6e-9)

    m_tx_start = extract_m(tx_start)
    m_tx_end = extract_m(tx_end)
    m_noop_start = extract_m(noop_start)
    m_noop_end = extract_m(noop_end)

    num_m = len(m_tx_start)

    m0_switch = abs(m_tx_end[0] - m_tx_start[0])
    m0_noop_switch = abs(m_noop_end[0] - m_noop_start[0])

    m_tx_switch_arr = [round(abs(m_tx_end[i] - m_tx_start[i])) for i in range(1, num_m)]
    m_tx_switch = sum(m_tx_switch_arr)
    m_noop_switch_arr = [round(abs(m_noop_end[i] - m_noop_start[i])) for i in range(1, num_m)]
    m_noop_switch = sum(m_noop_switch_arr)

    ratio_tx_noop = m_tx_switch / m_noop_switch
    ratio_tx_switch = m_tx_switch / m0_switch
    ratio_noop_switch = m_noop_switch / m0_switch
    ratio_tx_noop_switch = (m_tx_switch - m_noop_switch) / m0_switch

    print(f'm0 tx: {m0_switch}  ({m_tx_start[0]} -> {m_tx_end[0]})')
    print(f'm0 noop: {m0_noop_switch}  ({m_noop_start[0]} -> {m_noop_end[0]})')
    print('')
    print(f'm(tx):   {m_tx_start}')
    print(f'      -> {m_tx_end}')
    print('')
    print(f'm(noop): {m_noop_start}')
    print(f'      -> {m_noop_end}')
    print('')
    print(f'switched(tx): {m_tx_switch_arr}')
    print(f'switched(noop): {m_noop_switch_arr}')
    print('')
    print(f'tx/noop: {ratio_tx_noop:.3}')
    print(f'tx/m0: {ratio_tx_switch:.3}')
    print(f'noop/m0: {ratio_noop_switch:.3}')
    print(f'(tx-noop)/m0: {ratio_tx_noop_switch:.3}')

if __name__ == '__main__':
    extract_8xx(sys.argv[1])
