#!/usr/bin/env python3
"""
invoke with the path to a meas.csv file for the stacked_core 9xx demos
to extract higher-level info from them.
"""
import sys

from stacked_cores import load_csv, labeled_rows, last_row_before_t, extract_m

def extract_9xx(path: str):
    header, raw_rows = load_csv(path)
    rows = labeled_rows(header, raw_rows)

    tx_start = last_row_before_t(rows, 2e-9)
    tx_end = last_row_before_t(rows, 4e-9)
    rev_start = last_row_before_t(rows, 4e-9)
    rev_end = last_row_before_t(rows, 6e-9)

    m_tx_start = extract_m(tx_start)
    m_tx_end = extract_m(tx_end)
    m_rev_start = extract_m(rev_start)
    m_rev_end = extract_m(rev_end)

    m1_switch = abs(m_tx_end[1] - m_tx_start[1])
    m1_rev_switch = abs(m_rev_end[1] - m_rev_start[1])

    m_tx_switch_arr = [round(abs(m_tx_end[i] - m_tx_start[i])) for i in [0, 2]]
    m_tx_switch = sum(m_tx_switch_arr)
    m_rev_switch_arr = [round(abs(m_rev_end[i] - m_rev_start[i])) for i in [0, 2]]
    m_rev_switch = sum(m_rev_switch_arr)

    ratio_tx_switch = m_tx_switch / m1_switch
    ratio_roundtrip = m1_rev_switch / m1_switch

    print(f'm1 tx: {m1_switch}  ({m_tx_start[1]} -> {m_tx_end[1]})')
    print(f'm1 rev: {m1_rev_switch}  ({m_rev_start[1]} -> {m_rev_end[1]})')
    print('')
    print(f'm(tx):   {m_tx_start}')
    print(f'      -> {m_tx_end}')
    print('')
    print(f'm(rev): {m_rev_start}')
    print(f'      -> {m_rev_end}')
    print('')
    print(f'tx/m1: {ratio_tx_switch:.3}')
    print(f'rev/m1: {ratio_roundtrip:.3}')

if __name__ == '__main__':
    extract_9xx(sys.argv[1])
