use coremem::{self, Driver, AbstractSim};
use coremem::sim::spirv::{SpirvSim, WgpuBackend};
use coremem::sim::units::Frame;
use coremem::cross::mat::GenericMaterial;
use coremem::geom::Index;
use std::time::{Instant, Duration};

fn measure<F: FnMut()>(name: &str, n_times: u32, mut f: F) -> f32 {
    let mut times = vec![];
    for _ in 0..n_times {
        let start = Instant::now();
        f();
        let end = Instant::now();
        times.push(end - start);
    }
    let avg = (times.into_iter().sum::<Duration>() / n_times).as_secs_f32();
    println!("[{}]: {}s", name, avg);
    avg
}

fn measure_steps<S: AbstractSim + Clone + Default + Send + 'static>(name: &str, steps_per_call: u32, mut d: Driver<S::Real, S>) {
    measure(name, 100/steps_per_call, || d.step_until(Frame(steps_per_call.into())));
}

fn main() {
    coremem::init_logging();
    measure_steps("spirv/80", 1, Driver::new(
        SpirvSim::<f32, GenericMaterial<f32>, WgpuBackend>::new(Index::new(80, 80, 80), 1e-3)
    ));
    measure_steps("spirv/80 step(2)", 2, Driver::new(
        SpirvSim::<f32, GenericMaterial<f32>, WgpuBackend>::new(Index::new(80, 80, 80), 1e-3)
    ));
    measure_steps("spirv/80 step(10)", 10, Driver::new(
        SpirvSim::<f32, GenericMaterial<f32>, WgpuBackend>::new(Index::new(80, 80, 80), 1e-3)
    ));
    measure_steps("spirv/80 step(100)", 100, Driver::new(
        SpirvSim::<f32, GenericMaterial<f32>, WgpuBackend>::new(Index::new(80, 80, 80), 1e-3)
    ));
}

