use crate::geom::Line2d;
use coremem_cross::real::Real;
use coremem_cross::vec::Vec2;

#[derive(Clone, Debug, PartialEq)]
pub struct Polygon2d<R> {
    points: Vec<Vec2<R>>,
}

impl<R> Polygon2d<R> {
    pub fn new(points: Vec<Vec2<R>>) -> Self {
        Self {
            points
        }
    }
}

impl<R: Real> Polygon2d<R> {
    pub fn segments<'a>(&'a self) -> impl Iterator<Item=Line2d<R>> + 'a {
        (0..self.points.len()).into_iter().map(move |i| {
            let from = self.points[i];
            let to = *self.points.get(i+1).unwrap_or_else(|| &self.points[0]);
            Line2d::new(from, to)
        })
    }
    pub fn min_x(&self) -> R {
        self.points.iter().map(|p| p.x).reduce(|p, q| p.min_or_undefined(q)).unwrap()
    }

    pub fn max_x(&self) -> R {
        self.points.iter().map(|p| p.x).reduce(|p, q| p.max_or_undefined(q)).unwrap()
    }

    pub fn min_y(&self) -> R {
        self.points.iter().map(|p| p.y).reduce(|p, q| p.min_or_undefined(q)).unwrap()
    }

    pub fn max_y(&self) -> R {
        self.points.iter().map(|p| p.y).reduce(|p, q| p.max_or_undefined(q)).unwrap()
    }
}


