use crate::diagnostics::SyncDiagnostics;

use coremem_cross::mat::Material;
use coremem_cross::real::Real;
use coremem_cross::step::{SimMeta, StepEContext, StepHContext};
use coremem_cross::vec::{Vec3, Vec3u};

use super::SimBackend;

#[derive(Default)]
pub struct CpuBackend;

impl<R: Real, M: Material<R>> SimBackend<R, M> for CpuBackend {
    fn step_n(
        &mut self,
        _diag: &SyncDiagnostics,
        meta: SimMeta<R>,
        mat: &[M],
        stim_e: &[Vec3<R>],
        stim_h: &[Vec3<R>],
        e: &mut [Vec3<R>],
        h: &mut [Vec3<R>],
        m: &mut [Vec3<R>],
        num_steps: u32,
    ) {
        for _ in 0..num_steps {
            // step E field
            apply_all_cells(meta.dim(), |idx| {
                StepEContext::step_flat_view(meta, mat, stim_e, e, h, idx);
            });

            // step H field
            apply_all_cells(meta.dim(), |idx| {
                StepHContext::step_flat_view(meta, mat, stim_h, e, h, m, idx);
            });
        }
    }
}

fn apply_all_cells<F: FnMut(Vec3u)>(dim: Vec3u, mut f: F) {
    for z in 0..dim.z() {
        for y in 0..dim.y() {
            for x in 0..dim.x() {
                f(Vec3u::new(x, y, z));
            }
        }
    }
}

