use std::ops::Deref;

pub trait Time {
    fn to_frame(&self, time_step: f32) -> Frame;
    fn to_seconds(&self, time_step: f32) -> Seconds;
}

#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub struct Frame(pub u64);

impl Time for Frame {
    fn to_frame(&self, _: f32) -> Frame {
        *self
    }
    fn to_seconds(&self, time_step: f32) -> Seconds {
        Seconds((**self as f32) * time_step)
    }
}

impl Deref for Frame {
    type Target = u64;
    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

impl Frame {
    pub fn round_up(&self, div: u64) -> Frame {
        match self.0 % div {
            0 => *self,
            rem => Frame(self.0 + div - rem),
        }
    }
}

#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
pub struct Seconds(pub f32);

impl Time for Seconds {
    fn to_frame(&self, time_step: f32) -> Frame {
        Frame((**self / time_step).round() as u64)
    }
    fn to_seconds(&self, _: f32) -> Seconds {
        *self
    }
}

impl Deref for Seconds {
    type Target = f32;
    fn deref(&self) -> &Self::Target {
        &self.0
    }
}
