use spirv_std::RuntimeArray;

use coremem_cross::mat::Material;
use coremem_cross::real::Real;
use coremem_cross::step::{SimMeta, StepEContext, StepHContext};
use coremem_cross::vec::{Vec3, Vec3u};

use crate::support::SizedArray;

pub(crate) fn step_h<R: Real, M: Material<R>>(
    idx: Vec3u,
    meta: &SimMeta<R>,
    stimulus_h: &RuntimeArray<Vec3<R>>,
    material: &RuntimeArray<M>,
    e: &RuntimeArray<Vec3<R>>,
    h: &mut RuntimeArray<Vec3<R>>,
    m: &mut RuntimeArray<Vec3<R>>,
) {
    let dim = meta.dim();
    if idx.x() < dim.x() && idx.y() < dim.y() && idx.z() < dim.z() {
        let len = dim.product_sum_usize();

        let stim_h_array = unsafe { SizedArray::new(stimulus_h, len) };
        let mat_array = unsafe { SizedArray::new(material, len) };
        let e_array = unsafe { SizedArray::new(e, len) };
        let mut h_array = unsafe { SizedArray::new(h, len) };
        let mut m_array = unsafe { SizedArray::new(m, len) };

        StepHContext::step_flat_view(*meta, &mat_array, &stim_h_array, &e_array, &mut h_array, &mut m_array, idx);
    }
}

pub(crate) fn step_e<R: Real, M: Material<R>>(
    idx: Vec3u,
    meta: &SimMeta<R>,
    stimulus_e: &RuntimeArray<Vec3<R>>,
    material: &RuntimeArray<M>,
    e: &mut RuntimeArray<Vec3<R>>,
    h: &RuntimeArray<Vec3<R>>,
) {
    let dim = meta.dim();
    if idx.x() < dim.x() && idx.y() < dim.y() && idx.z() < dim.z() {
        let len = dim.product_sum_usize();

        let stim_e_array = unsafe { SizedArray::new(stimulus_e, len) };
        let mat_array = unsafe { SizedArray::new(material, len) };
        let mut e_array = unsafe { SizedArray::new(e, len) };
        let h_array = unsafe { SizedArray::new(h, len) };

        StepEContext::step_flat_view(*meta, &mat_array, &stim_e_array, &mut e_array, &h_array, idx);
    }
}

